import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

import 'controller.dart';
import 'popup.dart';
import 'progress.dart';
import 'snack.dart';
import 'toast.dart';

class MyDialog {
  BuildContext context;

  MyDialog.of(this.context);

  static const alignTop = const Alignment(0.0, -0.7);
  static const alignBottom = const Alignment(0.0, 0.7);

  static const iconSuccess = const Icon(
    CupertinoIcons.checkmark_circle_fill,
    color: Colors.green,
  );
  static const iconError =
      const Icon(CupertinoIcons.multiply_circle_fill, color: Colors.red);
  static const iconWarning = const Icon(
      CupertinoIcons.exclamationmark_triangle_fill,
      color: Colors.deepOrangeAccent);
  static const iconInfo = const Icon(CupertinoIcons.exclamationmark_circle_fill,
      color: Colors.blue);

  Future<bool?>? confirm(message,
      {String buttonText = 'OK',
      String title = '',
      String cancelText = 'Cancel'}) {
    late ModalController controller;
    Completer completer = Completer<bool>();
    controller = modal(
      message is Widget
          ? message
          : ListBody(
              children: message
                  .toString()
                  .split('\n')
                  .map<Widget>((item) => Text(item))
                  .toList()),
      [
        TextButton(
            onPressed: () {
              controller.close();
              completer.complete(false);
            },
            child: Text(cancelText)),
        ElevatedButton(
            onPressed: () {
              controller.close();
              completer.complete(true);
            },
            child: Text(buttonText)),
      ],
      title: title,
    ) as ModalController;
    controller.result = completer.future;

    return controller.result as Future<bool?>?;
  }

  Future<void> alert(message, {String buttonText = 'OK', String title = ''}) {
    late ModalController controller;
    Completer completer = Completer<bool>();
    controller = modal(
      message is Widget
          ? message
          : ListBody(
              children: message
                  .toString()
                  .split('\n')
                  .map<Widget>((item) => Text(item))
                  .toList()),
      [
        ElevatedButton(
          onPressed: () {
            controller.close();
            completer.complete();
          },
          child: Text(buttonText),
        ),
      ],
      title: title,
    ) as ModalController;

    return completer.future;
  }

  DialogController modal(Widget body, List<Widget> buttons,
      {String title = '', barrierDismissible = false}) {
    ModalController controller = ModalController(context);
    controller.result = showDialog<dynamic>(
      context: context,
      barrierDismissible: barrierDismissible,
      builder: (BuildContext context) {
        return AlertDialog(
          title: title.isEmpty ? null : Text(title),
          content: SingleChildScrollView(
            child: body,
          ),
          actions: buttons,
        );
      },
    );
    return controller;
  }

  DialogController popup(Widget body, {
    barrierDismissible = false,
    double height = 0,
    double borderRound = 10,
    EdgeInsetsGeometry padding = const EdgeInsets.all(10),
    Color barrierColor: const Color.fromRGBO(0, 0, 0, .6),
    Color backgroundColor: Colors.white,
    bool isDismissible: true,
    bool isScrollControlled: false,
    double? elevation,
    bool showClose = true,
    Widget closeButton = const Icon(Icons.cancel, color: Colors.black38,)
  }) {
    ModalController controller = ModalController(context);
    controller.result = showModalBottomSheet<dynamic>(
      backgroundColor: Colors.transparent,
      barrierColor: barrierColor,
      context: context,
      elevation: elevation,
      isDismissible: isDismissible,
      isScrollControlled: isScrollControlled,
      builder: (BuildContext context) {
        return PopupWidget(
          child: body,
          height:height,
          borderRound:borderRound,
          backgroundColor:backgroundColor,
          padding: padding,
          controller: controller,
          showClose: showClose,
          closeButton: closeButton,
        );
      },
    );

    return controller;
  }

  DialogController loading(String message,
      {showProgress = false, showOverlay = true, double time = 3}) {
    ValueNotifier<int> progressNotify = ValueNotifier<int>(0);
    ProgressController controller = ProgressController(context, progressNotify);
    controller.entry = OverlayEntry(builder: (context) {
      return ProgressWidget(
        notifier: progressNotify,
        showProgress: showProgress,
        showOverlay: showOverlay,
        message: message,
        controller: controller,
      );
    });

    controller.open();

    if (time > 0) {
      Future.delayed(Duration(milliseconds: (time * 1000).toInt())).then((v) {
        controller.close();
      });
    }
    return controller;
  }

  void toast(String message,
      {int duration = 2, Alignment align = alignTop, Icon? icon}) {
    OverlayEntry entry = OverlayEntry(builder: (context) {
      return ToastWidget(
        message,
        icon: icon,
        alignment: align,
        duration: duration,
      );
    });

    Overlay.of(context)!.insert(entry);
    Future.delayed(Duration(seconds: duration)).then((value) {
      // 移除层可以通过调用OverlayEntry的remove方法。
      entry.remove();
    });
  }

  EntryController snack(String message,
      {Widget? action,
      int duration = 3,
      Alignment align = alignBottom,
      double width = 0.7}) {
    ValueNotifier<int> progressNotify = ValueNotifier<int>(0);
    EntryController controller = EntryController(context, progressNotify);
    controller.entry = OverlayEntry(builder: (context) {
      return SnackWidget(
        message,
        alignment: align,
        action: action,
        duration: duration,
        notifier: progressNotify,
        controller: controller,
        maxWidth: width,
      );
    });

    controller.open();
    Future.delayed(Duration(seconds: duration)).then((value) {
      controller.close();
    });

    return controller;
  }
}
